package com.hjy.module.service.gather.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.utils.CommomUtils;
import com.hjy.common.utils.PageUtils;
import com.hjy.module.domain.gether.CbGather;
import com.hjy.module.domain.gether.CbGatherFee;
import com.hjy.module.domain.gether.CbGatherWarning;
import com.hjy.module.domain.norm.CbGive;
import com.hjy.module.domain.norm.CbNorm;
import com.hjy.module.domain.norm.CbNormFee;
import com.hjy.module.mapper.gather.CbGatherMapper;
import com.hjy.module.service.gather.ICbGatherFeeService;
import com.hjy.module.service.gather.ICbGatherService;
import com.hjy.module.service.gather.ICbGatherWarningService;
import com.hjy.module.service.norm.ICbGiveService;
import com.hjy.module.service.norm.ICbNormFeeService;
import com.hjy.module.service.norm.ICbNormService;
import com.hjy.module.vo.gather.CbGatherFeeVo;
import com.hjy.module.vo.gather.CbGatherVo;
import com.hjy.module.vo.gather.CbGatherWarningVo;
import com.hjy.module.vo.norm.CbNormFeeVo;
import com.hjy.module.vo.norm.CbNormVo;
import org.apache.commons.compress.utils.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.List;

/**
 * 数据采集Service业务层处理
 *
 * @author Tellsea
 * @date 2023-08-30
 */
@Service
public class CbGatherServiceImpl extends ServiceImpl<CbGatherMapper, CbGather> implements ICbGatherService {


    @Autowired
    private ICbGatherFeeService gatherFeeService;

    @Autowired
    private ICbNormService normService;

    @Autowired
    private ICbNormFeeService normFeeService;

    @Autowired
    private ICbGiveService giveService;

    @Autowired
    private ICbGatherWarningService warningService;

    @Override
    public TableDataInfo<CbGatherVo> queryList(CbGatherVo entity) {
        return PageUtils.buildDataInfo(this.baseMapper.queryList(PageUtils.buildPage(), entity));
    }

    @Override
    public List<CbGatherVo> queryAll(CbGatherVo entity) {
        return this.baseMapper.queryList(entity);
    }

    /**
     * 获取数据详情
     * @param id
     * @return
     */
    @Override
    public CbGatherVo queryById(String id) {
        CbGatherVo gatherVo = this.baseMapper.queryById(id);
        CbGatherFeeVo query = new CbGatherFeeVo();
        query.setGatherId(id);
        gatherVo.setFeeLists(gatherFeeService.queryAll(query));
        return gatherVo;
    }

    /**
     * 获取可采集的数据
     * @param id
     * @return
     */
    @Override
    public CbGatherVo getInfoByColl(String id){
        CbGatherVo gatherVo = this.baseMapper.queryById(id);
        List<CbGatherFeeVo> gatherFeeVoList = Lists.newArrayList();
        if (gatherVo.getType().equals("0")){ // 量化指标数据
           List<CbNorm> normList = normService.lambdaQuery().eq(CbNorm::getStatus,"1").eq(CbNorm::getDelFlag,"0").list();
           for (CbNorm norm : normList){
               List<CbNormFee> normFeeList = normFeeService.lambdaQuery()
                       .eq(CbNormFee::getStatus,"1")
                       .eq(CbNormFee::getDelFlag,"0")
                       .eq(CbNormFee::getNormId,norm.getId())
                       .list();
               for (CbNormFee normFee : normFeeList){
                   CbGatherFeeVo gatherFeeVo = new CbGatherFeeVo();
                   gatherFeeVo.setNormId(norm.getId());
                   gatherFeeVo.setNormName(norm.getName());
                   gatherFeeVo.setNormfeeId(normFee.getId());
                   gatherFeeVo.setNormfeeName(normFee.getFname());
                   gatherFeeVo.setGatherId(id);
                   gatherFeeVoList.add(gatherFeeVo);
               }
           }
        }else if (gatherVo.getType().equals("1")){ // 月报送数据
            List<CbGive> giveList = giveService.lambdaQuery().eq(CbGive::getStatus,"1").eq(CbGive::getDelFlag,"0").list();
            for (CbGive give : giveList){
                CbGatherFeeVo gatherFeeVo = new CbGatherFeeVo();
                gatherFeeVo.setNormfeeId(give.getId());
                gatherFeeVo.setNormfeeName(give.getFname());
                gatherFeeVo.setGatherId(id);
                gatherFeeVoList.add(gatherFeeVo);
            }
        }
        gatherVo.setFeeLists(gatherFeeVoList);
        return gatherVo;
    }

    /**
     * 保存采集数据
     * @param entity
     * @return
     */
    @Override
    public boolean saveEntity(CbGatherVo entity){
        entity.setCollStatus("1");
        this.baseMapper.updateById(entity);
        gatherFeeService.saveBatch(CommomUtils.copyList(entity.getFeeLists(), CbGatherFee.class));
        this.checkWraingData(entity); // 校验数据 生成预警信息
        return true;
    }

    /**
     * 修改采集数据
     * @param entity
     * @return
     */
    @Override
    public boolean updateEntity(CbGatherVo entity){
        this.baseMapper.updateById(entity);
        gatherFeeService.updateBatchById(CommomUtils.copyList(entity.getFeeLists(), CbGatherFee.class));
        this.checkWraingData(entity); // 校验数据 生成预警信息
        return true;
    }

    /**
     * 校验数据 生成预警信息
     * @param entity
     */
    public void checkWraingData(CbGatherVo entity) {
        List<CbGatherFeeVo> gatherFeeVos = entity.getFeeLists();
        if (entity.getType().equals("0")){ // 指标数据
            for (CbGatherFeeVo gatherFee : gatherFeeVos){
                CbNormFeeVo feeVo = normFeeService.queryById(gatherFee.getNormfeeId());
                BigDecimal collValue = gatherFee.getCollCalue(); // 采集值
                BigDecimal msValue = feeVo.getMeritsValue(); // 标准值
                BigDecimal msDtoValue = msValue.add(msValue.multiply(feeVo.getMsRetio()));  // 上公差
                BigDecimal msDtuValue = msValue.subtract(msValue.multiply(feeVo.getMsRetio())); // 下公差
                if (!bjdtodtu(collValue,msDtoValue,msDtuValue)){
                    // 数据异常->产生预警信息
                    CbGatherWarningVo warningVo = new CbGatherWarningVo();
                    warningVo.setType(entity.getType());
                    warningVo.setGetherId(entity.getId());
                    warningVo.setGetherfeeId(feeVo.getId());
                    warningVo.setErrValue(collValue);
                    warningService.save(warningVo);
                }
            }

        }else if (entity.getType().equals("1")){ // 报送数据
            for (CbGatherFeeVo gatherFee : gatherFeeVos){
                CbGive feeVo = giveService.queryById(gatherFee.getNormfeeId());
                BigDecimal collValue = gatherFee.getCollCalue(); // 采集值
                BigDecimal msValue = feeVo.getMeritsValue(); // 标准值
                BigDecimal msDtoValue = msValue.add(msValue.multiply(feeVo.getMsRetio()));  // 上公差
                BigDecimal msDtuValue = msValue.subtract(msValue.multiply(feeVo.getMsRetio())); // 下公差
                if (!bjdtodtu(collValue,msDtoValue,msDtuValue)){
                    // 数据异常->产生预警信息
                    CbGatherWarningVo warningVo = new CbGatherWarningVo();
                    warningVo.setType(entity.getType());
                    warningVo.setGetherId(entity.getId());
                    warningVo.setGetherfeeId(feeVo.getId());
                    warningVo.setErrValue(collValue);
                    warningService.save(warningVo);
                }
            }
        }
    }

    /**
     * 对比上下公差
     * @param collValue 实际值
     * @param msDtoValue 上公差
     * @param msDtuValue 下公差
     * @return
     */
    private static boolean bjdtodtu(BigDecimal collValue, BigDecimal msDtoValue, BigDecimal msDtuValue) {
        if (collValue.compareTo(msDtoValue) != 1 && collValue.compareTo(msDtuValue) != -1 ){
            return true;
        }
        return false;
    }



}
